# -*- coding: utf-8 -*-
"""
Created on Sun Dec 15 19:22:32 2013

@author: mel
"""
import os
import shutil
import time
import copy
import re
import wx
import model
import analytic
import app.resources as rc
from ctx import theContext as context
from model.writer import writer
import tran


class Project(model.TComponent):
    """Representation of C++ project"""

    folder_container = True
    diagram_container = True
    module_container = True

    def __init__(self, **kwargs):
        """Initialization of project"""
        self._dir = kwargs['dir']
        name = kwargs['name']
        self._language = kwargs.get('language', 'c++')
        self._includeDir = kwargs.get('includedir', "include")
        self._srcDir = kwargs.get('sourcedir', "src")
        self._useMaster = kwargs.get('usemaster', True)
        self._masterInclude = kwargs.get('masterinclude', name + ".h")
        self._author = kwargs.get('author', "<unknown>")
        self._date = kwargs.get('date', "08-10-2966")
        self._license = kwargs.get('license', None)
        self._type = kwargs.get('type', "unspecified")
        self._version = kwargs.get('version', [1, 0, 0])
        self._useMakefile = kwargs.get('usemakefile', False)
        self._types = kwargs.get('types', {})
        self._tasks = kwargs.get('tasks', [])
        self._contexts = kwargs.get('contexts', [])
        self._modified = True
        self._bookmarks = {}
        self._breakpoints = {}
        self._lastHdrTime = None
        super(Project, self).__init__(**kwargs)
        self.RegisterOutputDirs()
        if self._language == 'c++':
            import model.LibrariesFolder  # why I need this again?
            import model.cc  # why I need this again?
            self.namespace_container = True
            self.class_container = True
            self.package_container = False
            model.LibrariesFolder(parent=self, readonly=True)
            model.cc.TypesFolder(parent=self, readonly=True)
        else:
            import model.py  # why I need this again?
            self.namespace_container = False
            self.class_container = False
            self.package_container = True
            self.main_file = None
            self.py_file_pattern = re.compile('^(.)*.py$', re.I)
            # for python project, add hidden class object
            model.py.Class(parent=self, name='object',
                visibleInTree=False, declare=False, implement=False)
        #Add tasks control
        import model.tasks as tasks
        tasks.PendingTasks(parent=self)
        tasks.CurrentTasks(parent=self)
        tasks.FinishedTasks(parent=self)

    def breakpoints(self, model_file):
        """The breakpoints are stored using the uid of the object
        for reference"""
        uid = model_file.uid
        if uid not in self._breakpoints:
            self._breakpoints[uid] = {}
        return self._breakpoints[uid]

    def bookmarks(self, editable):
        """The bookmarks are stored using the uid of the object
        for reference. The editable objects may include ctors,
        dtors, standard methods or code files."""
        uid = editable.uid
        if uid not in self._bookmarks:
            self._bookmarks[uid] = {}
        return self._bookmarks[uid]

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['dir'] = self._dir
        kwargs['language'] = self._language
        kwargs['includedir'] = self._includedir
        kwargs['sourcedir'] = self._srcDir
        kwargs['usemaster'] = self._usemaster
        kwargs['masterinclude'] = self._masterinclude
        kwargs['author'] = self._author
        kwargs['date'] = self._date
        kwargs['license'] = self._license
        kwargs['type'] = self._type
        kwargs['version'] = self._version
        kwargs['usemakefile'] = self._useMakefile
        kwargs['types'] = self._types
        kwargs['tasks'] = self._tasks
        kwargs['contexts'] = copy.deepcopy(self._contexts)
        kwargs.update(super(Project, self).get_kwargs())
        return kwargs

    @property
    def types(self):
        """This method gets the list of visible types"""
        visible = lambda x: not x.inner_class or getattr(x, '_access', 'public') == 'public'
        return self(model.cc.Type, model.cc.Class, filter=visible, cut=True)

    def RegisterOutputDirs(self, logger=None):
        """Finds target generation directories and create it if missing."""
        if not os.path.exists(self._dir):
            os.makedirs(self._dir)
            if not os.path.exists(self._dir):
                e = "Project path missing: " + self._dir
                if logger is not None:
                    logger.AppendReportLine(e, wx.ICON_ERROR)
                wx.MessageBox(e, "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
                return False
        # for python projects, there are not other requiriments
        if self._language == 'python':
            return True
        sources_dir = os.path.join(self._dir, self._srcDir)
        headers_dir = os.path.join(self._dir, self._includeDir)
        sources_dir = os.path.realpath(sources_dir)
        headers_dir = os.path.realpath(headers_dir)
        if not os.path.exists(sources_dir):
            os.makedirs(sources_dir)
            if not os.path.exists(sources_dir):
                e = "Failed creating sources directory " + sources_dir
                if logger is not None:
                    logger.AppendReportLine(e, wx.ICON_ERROR)
                wx.MessageBox(e, "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
                return False
        if not os.path.exists(headers_dir):
            os.makedirs(headers_dir)
            if not os.path.exists(headers_dir):
                e = "Failed creating headers directory " + headers_dir
                if logger is not None:
                    logger.AppendReportLine(e, wx.ICON_ERROR)
                wx.MessageBox(e, "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
                return False
        self.sources_dir = sources_dir
        self.headers_dir = headers_dir
        return True

    def SortClasses(self):
        """Sort classes by inheritance level"""
        tc = [x for x in self.level_classes if not x.inner_library or x.inner_package]
        b = []
        index = 0
        while len(b) < len(tc):
            a = [x for x in tc if x not in b]
            for cls in a:
                for dcls in cls._deriv:
                    if dcls in a:
                        a.remove(dcls)
                # tambien consultamos las relaciones
                for toRel in cls(model.cc.RelationTo):
                    to = toRel._TO
                    if to in a:
                        a.remove(to)
            for cls in a:
                cls._inheritance_level = index
            b += a
            index = index + 1
        tc.sort(key=model.cc.Class.lower)
        for cls in tc:
            delattr(cls, '_inheritance_level')
        self.sorted_classes = tc

    def WriteTimestamp(self, pf, logger=None):
        """Write the timestamp and other info"""
        pf.writeln('/***')
        pf.writenl()
        pf.writeln(' File    :{}'.format(self._masterInclude))
        pf.writeln(' Created :{}'.format(time.strftime("%d/%m/%Y %H:%M:%S")))
        pf.writeln(' Author  :{}'.format(self._author))
        if self._license:
            pf.writenl()
            pf.writeln(' Licensed under {}'.format(self._license))
        pf.writenl()
        pf.writeln('***/')
        pf.writenl()

    def WriteForwards(self, pf, logger=None):
        """Write the forward class declarations"""
        pf.writeln('//forward references')
        for cls in self.sorted_classes:
            if cls.is_external:
                continue
            pf.writeln('{decl};'.format(decl=cls.reference))
        pf.writenl()

    def WriteMakefile(self, logger=None):
        """Write the makefile"""
        #open the file
        makefile = os.path.join(self._dir, 'Makefile')
        makefile = os.path.realpath(makefile)
        f = open(makefile, 'w')
        pf = writer.for_file(f)
        # write a very quick makefile
        # a ) definitions
        pf.writeln('# definitions')
        pf.writenl()
        pf.writeln('CC = g++')
        pf.writeln('AR = ar')
        pf.writeln('CFLAGS = -ggdb -fexceptions -Wall -std=c++0x -fPIC')
        pf.writeln('INCLUDE = -I{self._includeDir}'.format(self=self))
        pf.writeln('SRC = {self._srcDir}'.format(self=self))
        pf.writenl()
        # make targets all/clean
        # all : first, the prerequisites
        if self._type == "static library":
            product = self.name
        elif self._type == "dynamic library":
            product = '{name}.so'.format(name=self.name)
        elif self._type == "static library":
            product = 'lib{name}.a'.format(name=self.name)
        elif self._type == "executable":
            product = '{name}'.format(name=self.name)
        pf.writeln('all : {product}'.format(product=product))
        #master target
        pf.writenl()
        self.SortClasses()
        prerequisites = ' '.join(['{name}.o'.format(name=x.name) for x in self.sorted_classes if not x.is_external])
        pf.writeln('{product}: {prerequisites}'.format(product=product, prerequisites=prerequisites))
        if self._type == 'static library':
            pf.writeln('\t$(AR) -r -s ./{product} {prerequisites}'.format(
                product=product, prerequisites=prerequisites))
        elif self._type == 'dynamic library':
            pf.writeln('\t$(CC) -shared  {prerequisites}  -o {product}'.format(
                product=product, prerequisites=prerequisites))
        elif self._type == 'executable':
            pf.writeln('\t$(CC) {prerequisites}  -o {product}'.format(
                product=product, prerequisites=prerequisites))
        pf.writenl()
        # target files
        for cls in self.sorted_classes:
            if cls.is_external:
                continue
            pf.writeln('{name}.o: {src}/{name}.cpp {inc}/{name}.h'.format(name=cls.name,
                inc=self._includeDir, src=self._srcDir))
            pf.writeln('\t$(CC) $(CFLAGS) $(INCLUDE) -c {src}/{name}.cpp -o {name}.o'.format(src=self._srcDir,
                name=cls.name))
            pf.writenl()
        # clean
        pf.writeln('clean:')
        pf.writeln('\trm *.o {product}'.format(product=product))
        pf.writenl()
        f.close()

    def WriteMasterHeader(self, force=False, logger=None):
        """Write the master include file"""
        #open the file
        fname = os.path.join(self.headers_dir, self._masterInclude)
        regenerate = False
        if force or not os.path.exists(fname) or self._lastHdrTime is None:
            regenerate = True
        elif os.path.getmtime(fname) < self._lastHdrTime:
            regenerate = True
        if not regenerate:
            return True
        f = open(fname, 'w')
        pf = writer.for_file(f)

        #write a timestamp and version information
        self.WriteTimestamp(pf)

        safe_name = self._name.upper().replace('.','_')

        #write include safeward
        pf.writeln('#if !defined({name}_H_INCLUDED)'.format(name=safe_name))
        pf.writeln('#define {name}_H_INCLUDED'.format(name=safe_name))
        pf.writenl()

        #write the custom definitions
        if len(self._contexts) > 0:
            for item in self._contexts:
                if item._enable and len(item._define):
                    pf.writeln('/**')
                    pf.writeln('context {name}:'.format(name=item._name))
                    pf.writeln('{note}'.format(note=item.note))
                    pf.writeln('**/')
                    pf.writenl()
                    pf.writeln('{}'.format(item._define))
                    pf.writenl()

        #write custom types
        for t in self(model.cc.Type, filter=lambda x: type(x.parent) is model.cc.TypesFolder):
            if t._readOnly:
                continue
            if len(t._definition.strip()) == 0:  # skip empty declarations
                continue
            pf.writeln('//type definition for {}'.format(t._name))
            pf.writeln('{}'.format(t._definition))
            pf.writenl()

        #write forward references
        self.WriteForwards(pf)

        #write prolog user definitions
        #self.WriteUserProlog()

        #write clases includes without inlines
        for cls in self.sorted_classes:
            if cls.is_external:
                continue
            pf.writeln('#include "{name}.h"'.format(name=cls._name))
        pf.writenl()
        pf.writeln('#define INCLUDE_INLINES')

        #write class includes again for inlines
        for cls in self.sorted_classes:
            if cls.is_external:
                continue
            pf.writeln('#include "{name}.h"'.format(name=cls._name))

        #write modules includes
        pf.writenl()
        for module in self.modules:
            if module.inner_library:
                continue
            pf.writeln('#include "{name}.h"'.format(name=module._header))

        #write end safeguard
        pf.writenl()
        pf.writeln('#endif //{name}_H_INCLUDED'.format(name=safe_name))
        pf.writenl()
        f.close()
        self._lastHdrTime = os.path.getmtime(fname)
        return True

    @tran.DelayedMethod
    def ExportCppCodeFiles(self, force=False, logger=None):
        """Do export c++ files"""
        # create sorted main classes list
        self.SortClasses()
        # create the master include file?
        if self._useMaster:
            if not self.WriteMasterHeader(force, logger):
                return
        if self._useMakefile:
            self.WriteMakefile()
        # generate through main classes
        for obj in self.sorted_classes:
            if obj.is_external:
                continue
            obj.updateSources(force)
            if logger is not None:
                e = "Generate source for class " + obj._name
                logger.AppendReportLine(e, wx.OK)
            obj.updateHeaders(force)
            if logger is not None:
                e = "Generate include for class " + obj._name
                logger.AppendReportLine(e, wx.OK)

        # remove temps
        delattr(self, 'sorted_classes')
        # generate through modules
        for obj in [x for x in self.modules if not x.inner_library]:
            obj.updateSources(force)
            if logger is not None:
                e = "Generate source for class " + obj._name
                logger.AppendReportLine(e, wx.OK)
            obj.updateHeaders(force)
            if logger is not None:
                e = "Generate include for class " + obj._name
                logger.AppendReportLine(e, wx.OK)

    @property
    def dir(self):
        """return the project directory"""
        return self._dir

    def OnUndoRedoRemoving(self):
        """Handle OnUndoRedoRemoving, prevent generating files"""
        stack = tran.TransactionStack
        if self._language == 'c++':
            method = self.ExportCppCodeFiles.inner
        else:
            method = self.ExportPythonCodeFiles.inner
        if method not in stack.delayedCallsFiltered:
            stack.delayedCallsFiltered.append(method)
        super(Project, self).OnUndoRedoRemoving()
        context.app.RemoveProject(self)

    @tran.DelayedMethod
    def ExportPythonCodeFiles(self, force=False, logger=None):
        """Do export python files"""
        logger and logger.AppendReportLine('exporting code files', wx.ICON_INFORMATION)
        for obj in self.packages:
            if obj.parent.inner_package:
                continue
            obj.ExportPythonCodeFiles(force, logger)
        for obj in self.modules:
            if obj.parent.inner_package:
                continue
            obj.ExportPythonCodeFiles(force, logger)

    def ExportCodeFiles(self, force=False, logger=None):
        """Exports all the source code for the project"""
        # create target directories if needed
        if not self.RegisterOutputDirs(logger):
            return
        if self._language == 'c++':
            self.ExportCppCodeFiles(force, logger)
        else:
            self.ExportPythonCodeFiles(force, logger)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        context.app.AddProject(self)
        super(Project, self).OnUndoRedoAdd()

    def GetTabBitmap(self):
        """Get the bitmap for tab control"""
        if self._language == 'c++':
            return rc.GetBitmap('cppproject')
        else:
            return rc.GetBitmap('python')

    @property
    def bitmap_index(self):
        """Index of tree image"""
        if self._language == 'c++':
            return rc.GetBitmapIndex('cppproject')
        else:
            return rc.GetBitmapIndex('python')

    @property
    def buildable(self):
        """The project may be built?"""
        return self._language in ['c++']

    def __getstate__(self):
        """Get the pickle context. The projects are always
        stored with independence of the workspace container"""
        state = dict(self.__dict__)
        state['_parent'] = None
        return state

    def __setstate__(self, d):
        """In general, if we change something in any class,
        the serialization of old model objects requires this
        filter. When the model handles serialization in an autommated
        fashion, this would be automatically done, based on element
        version."""
        if '_bookmarks' not in d:
            d['_bookmarks'] = {}
        if '_breakpoints' not in d:
            d['_breakpoints'] = {}
        # Do we need to store expressions? At first, it looks interesting,
        # but, what happens if some evil expression hangs our debugging sessions?
        # It's better to provide a facility for loading/storing debugger environment
        # later.
        self.__dict__ = d

    def SetModified(self, bModified=True):
        """Set the modified state"""
        self._lastHdrTime = None
        self._modified = bModified

    def IsModified(self):
        """Get the modified state"""
        return self._modified

    def import_python_dir(self, parent, path, progress=None):
        """Import the contents of the path into the project.
        The parent argument must be the self project or any existent package.
        The contents of the path and recursively scanned subdirs will be copied
        into the directory corresponding to the parent.
        This method will fail if path belongs to the project."""

        # check that path is not inside parent
        target = os.path.realpath(parent.dir)
        path = os.path.realpath(path)
        if len(path) >= len(target) and target == path[:len(target)]:
            raise ValueError('path must be not contained in target')

        for element in os.listdir(path):
            #skip hidden elements
            if element[0] == '.':
                continue
            if len(element) > 1:
                if element[-2:] == '.o':
                    continue
                if len(element) > 3:
                    if element[-4:] in ['.pyc', '.obj', '.sbr']:
                        continue
            abspath = os.path.join(path, element)
            if progress:
                progress.m_text.SetValue('working now on {}'.format(abspath))
                wx.YieldIfNeeded()
            if os.path.isdir(abspath):
                #skip __pycache__
                if element == '__pycache__':
                    continue
                new_directory = os.path.join(target, element)
                try:
                    if not os.path.exists(new_directory):
                        os.makedirs(new_directory)
                except:
                    # very bad thing happens
                    wx.MessageBox('failed creating dir {}'.format(new_directory))
                    continue
                #create a package and iterate
                kwargs = {'parent': parent, 'name': element,
                    'noinit': True}
                package = model.py.Package(**kwargs)
                self.import_python_dir(package, abspath, progress)
                continue
            if not os.path.isfile(abspath):
                # may be a link or a block device ... skip
                continue
            # Ok, create the same file
            new_file = os.path.join(target, element)
            if os.path.exists(new_file):
                # since the file already exists, we must ask user
                choice = wx.MessageBox(
                    'The file {} already exists. Overwrite it?'.format(element),
                    'Attention', wx.YES_NO | wx.CANCEL | wx.ICON_EXCLAMATION, parent=context.frame)
                if choice == wx.NO:
                    continue
                if choice == wx.CANCEL:
                    return False
            # copy the file
            try:
                shutil.copyfile(abspath, new_file)
            except:
                # very bad thing happens
                wx.MessageBox('failed creating file {}'.format(new_file))
                continue
            if self.py_file_pattern.match(element):
                #create new module
                if progress:
                    progress.m_text.SetValue('import {}'.format(abspath))
                    wx.YieldIfNeeded()
                with open(abspath, "r") as module_file:
                    content = module_file.read()
                kwargs = {'parent': parent, 'name': element[:-3],
                    'content': content}
                module = model.py.Module(**kwargs)
                if not module.analize():
                    module.Delete()
        return True

    def update_from_sources(self, **kwargs):
        """read sources and update model"""
        src_dir = kwargs.get('dir', self._dir)  # this is the root dir
        src_dir = os.path.abspath(src_dir)
        target_dir = os.path.abspath(self._dir)
        # ensure the dir exists
        import traceback
        import sys
        try:
            if not os.path.exists(target_dir):
                os.makedirs(target_dir)
            progress = kwargs.get('progress', None)
            if self._language == 'python':
                analytic.pyparse.set_project(self)
                if self.import_python_dir(self, src_dir, progress):
                    # refresh vincled files
                    self.RefreshProjectFiles()
        except Exception, e:
            wx.MessageBox('Failed : {}'.format(e), 'Error', wx.OK | wx.ICON_ERROR)
            traceback.print_exc(file=sys.stdout)
            print type(e)     # the exception instance
            print e.args      # arguments stored in .args
            print e
            return False
        return True

    def RefreshProjectFiles(self):
        """Refresh the files contained in a sngle projec"""
        # The project itself represents the root directory
        # so we need to scan the root dir
        _dir = os.path.realpath(self._dir)
        if not os.access(_dir, os.R_OK):
            return False
        # create a temporal dictionnary
        r = os.path.realpath
        j = os.path.join
        if self._language == 'c++':
            plc = self.level_classes
            s = self.sources_dir
            self.cpp_set = dict([(r(j(s, '{}.cpp'.format(c._name))), c) for c in plc])
            s = self.headers_dir
            self.h_set = dict([(r(j(s, '{}.h'.format(c._name))), c) for c in plc])
        elif self._language == 'python':
            self.file_dict = dict([(r(j(x.dir, '{x._name}.py'.format(x=x))), x)
                for x in self.modules])
            self.file_dict.update([(r(j(x.dir, '__init__.py')), x)
                for x in self.packages])

        # list all files excludding hidden files
        hidden = re.compile(r'\.(.)*')
        project = re.compile(r'(.)*\.pcc')
        excluded = [hidden, project]
        _fset = [s for s in os.listdir(_dir) if not any(y.match(s) for y in excluded)]
        kwargs = {'parent': self}
        knowns = self[model.file.File] + self[model.file.Dir]
        name_knowns = dict([(x.abs_file, x) for x in knowns])
        for f in _fset:
            element = os.path.join(_dir, f)
            if element in name_knowns:
                name_knowns[element].Refresh()
                del name_knowns[element]
                continue
            kwargs['file'] = element
            if os.path.isdir(element):
                model.file.Dir(**kwargs).Refresh()
            else:
                model.file.File(**kwargs).Refresh()
        # remove unexistent file
        for k in name_knowns:
            name_knowns[k].Delete()
        if self._language == 'c++':
            del self.cpp_set
            del self.h_set
        elif self._language == 'python':
            del self.file_dict





"""Subclass of codeNavigator, which is generated by wxFormBuilder."""

import copy
import wx
import app
import app.resources as rc
import model


# Implementing codeNavigator
class CodeNavigator(app.codeNavigator):
    """Dialog for code insertion"""
    def __init__(self, parent, method):
        """Initialice dialog"""
        self.method = method
        super(CodeNavigator, self).__init__(parent)

    def add_root(self, name, _type):
        """Add element to tree"""
        # we need to find all visible methods
        self.m_treeCtrl3.AppendItem(self.root, name, data=wx.TreeItemData(_type))

    def ShowModal(self):
        """Initialize dialog and go"""
        # add root
        self.root = self.m_treeCtrl3.AddRoot(wx.EmptyString)
        # insert methods for variables
        t = model.cc.typeinst(type=self.method.inner_class, ptr=True, constptr=True)
        self.add_root('this', t)

        super(wx.Dialog, self).Show()  # skip dialog modeless handling but show window
        return super(CodeNavigator, self).ShowModal()

    def OnKeyDown(self, event):
        if event.GetKeyCode() == wx.WXK_ESCAPE:
            self.EndModal(wx.ID_CANCEL)

    # Handlers for codeNavigator events.
    def OnSelectedItem(self, event):
        """Handle select item"""
        # TODO: Implement OnSelectedItem
        pass

    def OnExpandItem(self, event):
        # TODO: Implement OnExpandItem
        """Handle select item"""
        pass





"""Subclass of PdfViewPane, which is generated by wxFormBuilder."""
import wx
import app
from app.ui.pane import BasePane

# Implementing PdfViewPane
class PdfViewPane(BasePane, app.PdfViewPane):
    """Pane for visualize pdf documents"""
    def __init__(self, parent, mainframe, fileobj):
        """Init"""
        super(PdfViewPane, self).__init__(parent)
        self.m_pdfView.UsePrintDirect = False
        self.m_pdfView.SetFont( wx.Font( wx.NORMAL_FONT.GetPointSize(), 70, 94, 90, False, "Arial" ) )
        self.m_pdfView.SetForegroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_3DDKSHADOW ) )
        self.m_pdfView.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_BACKGROUND ) )
        #self.m_pdfView.SetSizerProps(expand=True, proportion=1)
        self.m_pdfView.LoadFile(fileobj._file)
        self.m_pdfView.Show()
        self.m_pdfView.Update()




# -*- coding: utf-8 -*-

"""Subclass of PyModulePane, which is generated by wxFormBuilder."""

#import wx
import wx
import wx.stc
import app
from ctx import theContext as context
import tran
from app.ui import pane
from activity.models.ui import ui as ui

CLOSE_PANE_ID = wx.ID_HIGHEST + 100


# Implementing PyModulePane
class PyModulePane(ui.PyModulePane):
    """Implements method pane editor"""

    #command ids: run
    _saveCode = wx.Window.NewControlId()
    _undoCode = wx.Window.NewControlId()
    _redoCode = wx.Window.NewControlId()
    _copyCode = wx.Window.NewControlId()
    _cutCode = wx.Window.NewControlId()
    _pasteCode = wx.Window.NewControlId()
    _deleteCode = wx.Window.NewControlId()
    _toggleFull = wx.Window.NewControlId()
    _navigator = wx.Window.NewControlId()

    def _create_toolbars(self):
        """Create custom toolbar"""
        self.m_localToolbar = wx.aui.AuiToolBar(self, wx.ID_ANY, wx.DefaultPosition,
            wx.DefaultSize, wx.aui.AUI_TB_HORZ_LAYOUT)
        self.m_save = self.m_localToolbar.AddTool(self._saveCode,
            u"save project", wx.ArtProvider.GetBitmap(wx.ART_FILE_SAVE, wx.ART_TOOLBAR),
            wx.NullBitmap, wx.ITEM_NORMAL, u"Save", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_undo = self.m_localToolbar.AddTool(self._undoCode, u"tool",
            wx.ArtProvider.GetBitmap(wx.ART_UNDO, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Undo", wx.EmptyString, None)
        self.m_redo = self.m_localToolbar.AddTool(self._redoCode, u"tool",
            wx.ArtProvider.GetBitmap(wx.ART_REDO, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Redo", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_copy = self.m_localToolbar.AddTool(self._copyCode, u"copy",
            wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Copy", wx.EmptyString, None)
        self.m_cut = self.m_localToolbar.AddTool(self._cutCode, u"cut", wx.ArtProvider.GetBitmap(
            wx.ART_CUT, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL, u"Cut", wx.EmptyString, None)
        self.m_paste = self.m_localToolbar.AddTool(self._pasteCode, u"paste",
            wx.ArtProvider.GetBitmap(wx.ART_PASTE, wx.ART_TOOLBAR), wx.NullBitmap,
            wx.ITEM_NORMAL, u"Paste", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_delete = self.m_localToolbar.AddTool(self._deleteCode, u"delete",
            wx.ArtProvider.GetBitmap(wx.ART_DELETE, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Delete", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_fullScreen = self.m_localToolbar.AddTool(self._toggleFull, u"tool",
            wx.ArtProvider.GetBitmap(u"gtk-fullscreen", wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Toggle fullscreen", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_navigator = self.m_localToolbar.AddTool(self._navigator, u"tool", wx.Bitmap(u"app/res/method.xpm",
            wx.BITMAP_TYPE_ANY), wx.NullBitmap, wx.ITEM_NORMAL, u"insert call", wx.EmptyString, None)
        self.m_localToolbar.Realize()
        self.m_toolbarSizer.Add(self.m_localToolbar, 0, wx.ALL, 5)
        self.Layout()
        # Connect Events
        self.m_editor.Bind(wx.EVT_KILL_FOCUS, self.OnLeaveFocus)
        self.m_editor.Bind(wx.EVT_SET_FOCUS, self.OnGetFocus)

    def __init__(self, parent, mainframe, method):
        """Intialization of method editor"""
        super(PyModulePane, self).__init__(parent)
        self._mainframe = mainframe
        self._object = method
        self._types = {}
        self._notebook = parent

        self._create_toolbars()
        self.m_editor.SetText(method._content)
        self.m_editor.Initialize(self._types)
        """
        if method.note:
            self.m_editor.AnnotationSetVisible(wx.stc.STC_ANNOTATION_BOXED)
            self.m_editor.AnnotationSetText(0, method.note)"""
        self.m_editor.SetReadOnly(method._readOnly)
        #self.m_editor.SendMsg(2540, wp="Prueba de anotacion", lp=long(0))
        #self.m_editor.SendMsg(2548, wp=1, lp=long(0))
        self.Layout()
        wx.stc.EVT_STC_CHANGE(self.m_editor,
            self.m_editor.GetId(), self.OnEditorChange)

        #self.Bind(EVT_STC_STYLENEEDED, self.handleStyleNeeded,
        #id=self.m_editor.GetId())
        frame = context.frame

        self.Bind(wx.EVT_MENU, frame.CloseCurrentDocPane, id=CLOSE_PANE_ID)
        self.Bind(wx.EVT_MENU, self.OnDeleteCode, id=self._deleteCode)
        self.Bind(wx.EVT_MENU, self.OnUndoCode, id=self._undoCode)
        self.Bind(wx.EVT_MENU, self.OnRedoCode, id=self._redoCode)
        self.Bind(wx.EVT_MENU, self.OnCopyCode, id=self._copyCode)
        self.Bind(wx.EVT_MENU, self.OnPasteCode, id=self._pasteCode)
        self.Bind(wx.EVT_MENU, self.OnCutCode, id=self._cutCode)
        self.Bind(wx.EVT_MENU, self.OnSaveCode, id=self._saveCode)
        self.Bind(wx.EVT_MENU, self.OnFullScreen, id=self._toggleFull)

        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateDeleteCode, id=self._deleteCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUndoCode, id=self._undoCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateRedoCode, id=self._redoCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateCopyCode, id=self._copyCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdatePasteCode, id=self._pasteCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateCutCode, id=self._cutCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateSaveCode, id=self._saveCode)

        self.m_editor.popup_handler = self
        self._fullScreen = None

        aTable = wx.AcceleratorTable([
            wx.AcceleratorEntry(wx.ACCEL_ALT | wx.ACCEL_CTRL, ord('F'), self._toggleFull),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('W'), CLOSE_PANE_ID),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('S'), self._saveCode),
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_DELETE, self._deleteCode)
        ])
        self.SetAcceleratorTable(aTable)

    def Refresh(self):
        """Update editor from external changes (like undo/redo)"""
        self.m_editor.SetText(self._object._content)
        if wx.__version__ >= '3.0.0':
            self.m_editor.SetModified(False)

    def PreDelete(self):
        """Remove toolbar first in order to avoid gtk-collision at close"""
        #self.m_editor.Destroy()
        self.m_localToolbar.Destroy()

    def Popup(self, editor, text=None):
        """Shows a custom popup"""
        menu = wx.Menu()
        if text:
            # in the future, we need to show a complete floating dialog here
            # we receive "this->" or "object." here ..
            pass
        else:
            menu.Append(self._mainframe.editProperties.GetId(),
            "test menu", "Edit relation properties")
        pos = editor.GetCurrentPos()
        p = editor.PointFromPosition(pos)
        editor.PopupMenu(menu, p)
        menu.Destroy()

    def OnNavigator(self, event):
        """Navigate throug the class structure"""
        pos = self.m_editor.GetCurrentPos()
        p = self.m_editor.PointFromPosition(pos)
        p = self.m_editor.ClientToScreen(p)
        dialog = app.ui.dlg.CodeNavigator(self, self._object)
        dialog.Move(p)
        if dialog.ShowModal() != wx.ID_OK:
            return
        pass

    def OnUndoCode(self, event):
        """Edit undo"""
        self.m_editor.Undo()

    def OnRedoCode(self, event):
        """Edit redo"""
        self.m_editor.Redo()

    def OnUpdatePasteCode(self, event):
        """Edit paste?"""
        event.Enable(self.m_editor.CanPaste())

    def OnUpdateCopyCode(self, event):
        """Edit copy?"""
        (b, e) = self.m_editor.GetSelection()
        event.Enable(b != e)

    def OnUpdateCutCode(self, event):
        """Edit cut?"""
        if self._object._readOnly:
            event.Enable(False)
        else:
            (b, e) = self.m_editor.GetSelection()
            event.Enable(b != e)

    def OnUpdateRedoCode(self, event):
        """Edit redo?"""
        if self.m_editor.CanRedo():
            event.SetText("Redo edit")
            event.Enable(True)
        else:
            event.SetText("Can't redo")
            event.Enable(False)

    def OnUpdateUndoCode(self, event):
        """Edit undo?"""
        if self.m_editor.CanUndo():
            event.SetText("Undo edit")
            event.Enable(True)
        else:
            event.SetText("Can't undo")
            event.Enable(False)

    def OnUpdateDeleteCode(self, event):
        """Edit delete?"""
        event.Enable(not self._object._readOnly)

    def Enabled(self, event):
        """Joker event"""
        event.Enable(True)

    def Disabled(self, event):
        """Joker event"""
        event.Enable(False)

    def UpdateModifiedTitle(self):
        """Update the tile after modifications"""
        if not self._fullScreen:
            i = self._notebook.GetPageIndex(self)
            s = self._notebook.GetPageText(i)
            m = '[modified] '
            if self.m_editor.IsModified():
                if s.find(m) == wx.NOT_FOUND:
                    self._notebook.SetPageText(i, m + s)
            else:
                if s.find(m) != wx.NOT_FOUND:
                    s = s.replace(m, '')
                    self._notebook.SetPageText(i, s)

    def OnEditorChange(self, event):
        """Called when editor status change"""
        if not self._fullScreen:
            self.UpdateModifiedTitle()

    @tran.TransactionalMethod('edit method')
    def Commit(self):
        """Update document"""
        text = self.m_editor.GetText()
        o = self._object
        if text == o._content:
            return False
        o.SaveState()
        o._content = text
        o.project.SetModified(True)
        o.ExportPythonCodeFiles()
        return True

    def OnDeleteCode(self, event):
        """Handle delete key"""
        (b, e) = self.m_editor.GetSelection()
        if b == e:
            self.m_editor.CharRight()
        self.m_editor.DeleteBack()

    def OnCopyCode(self, event):
        """Handle a copy event"""
        self.m_editor.Copy()

    def OnPasteCode(self, event):
        """Handle a paste event"""
        self.m_editor.Paste()

    def OnCutCode(self, event):
        """Handle a paste event"""
        self.m_editor.Cut()

    def OnUpdateSaveCode(self, event):
        """Handles update event"""
        event.Enable(self.m_editor.IsModified())

    def OnSaveCode(self, event):
        """Handles save event"""
        self.Commit()
        self.m_editor.ResetModified()
        self.UpdateModifiedTitle()

    def OnFullScreen(self, event):
        """ShowFullscreen"""
        if self._fullScreen:
            self._fullScreen.LeaveFullScreen()
            wx.YieldIfNeeded()
            self._fullScreen.Close(True)
            self._fullScreen = None
            self.UpdateModifiedTitle()
        else:
            self._fullScreen = pane.FullScreen(self)
        self.m_editor.SetFocus()

    def handleStyleNeeded(self, event):
        """Handle scintylla events"""
        wnd = self.m_editor
        pos = wnd.GetCurrentPos()
        if pos == 0:
            return
        if wnd.GetCharAt(pos - 1) == wx.T(':'):
            lnno = wnd.GetCurrentLine()
            iden = wnd.GetLineIndentation(lnno - 1)
            wnd.SetLineIndentation(lnno, iden)

            # = self.m_editor.GetFoldLevel(lnno - 1)
        # if this is not a normal line
        #self.m_editor.SetFoldLevel(lnno,LastIndent)
        #pass


    #def AutoIdent(self, event):
    #    """Handle enter"""
    #    key = event.GetKeyCode()
    #    wnd = self.m_editor
    #    if key == wx.WXK_NUMPAD_ENTER or key == wx.WXK_RETURN:
    #        if wnd.GetCharAt(wnd.GetCurrentPos() - 1) == 58:
    #            line = wnd.GetCurrentLine() + 1
    #            wnd.SetLineIndentation(line,
    #                wnd.GetLineIndentation(line) + wnd.GetTabWidth())
    #    event.Skip()






"""Subclass of TasksPane, which is generated by wxFormBuilder."""

import wx
import app


# Implementing TasksPane
class TasksPane(app.TasksPane):
    """Implements task pane"""
    def __init__(self, parent):
        """"""
        super(TasksPane, self).__init__(parent)



"""Subclass of Breakpoints, which is generated by wxFormBuilder."""
import wx
from activity.files.ui import ui as ui


# Implementing Breakpoints
class Breakpoints(ui.Breakpoints):
    """Pane showing breakpoints"""
    def __init__(self, parent, handler):
        """Initialize pane"""
        self._handler = handler
        super(Breakpoints, self).__init__(parent)
        img = wx.ImageList(24, 24, True, 1)
        img.Add(wx.Bitmap(u"app/res/breakpoint.xpm", wx.BITMAP_TYPE_ANY))
        self.m_listBreakpoints.AssignImageList(img, wx.IMAGE_LIST_SMALL)
        sz = self.GetClientSize()
        self.m_listBreakpoints.InsertColumn(0, '', wx.LIST_FORMAT_LEFT, sz.x)

    def UpdateData(self):
        """Update the data"""
        self.m_listBreakpoints.DeleteAllItems()
        breakpoints = self._handler.breakpoints
        keys = breakpoints.keys()
        keys.sort()
        for i in range(0, len(keys)):
            key = keys[i]
            bpi = breakpoints[key]
            text = '#{num} at {file}:{line}'.format(
                num=str(key),
                file=bpi[0],
                line=bpi[1])
            self.m_listBreakpoints.InsertStringItem(i, text, 0)

    # Handlers for Breakpoints events.
    def OnSelectBreakpoint(self, event):
        """Enable/Disable breakpoint"""
        pass

    def OnEditBreakpoint(self, event):
        """Edit breakpoint conditions"""
        # TODO: Implement OnEditBreakpoint
        pass

    def OnBreakpoinMenu(self, event):
        """Show popup menu for breakpoints"""
        # TODO: Implement OnBreakpoinMenu
        pass

    def OnSize(self, event):
        """Resize listctrl"""
        self.m_listBreakpoints.SetColumnWidth(0, event.m_size.x)




"""Subclass of NewPyMember, which is generated by wxFormBuilder."""
import re
import wx
import wxx
from activity.models.ui import ui as ui
import app.resources as rc


# Implementing NewPyMember
class PyMemberDialog(ui.NewPyMember):
    """This dialog allows to specify the python class
    member attributes."""
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialice dialog"""
        super(PyMemberDialog, self).__init__(parent)
        self._container = container
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("py_member"))
        self.SetIcon(icon)

    # Handlers for NewPyMember events.
    def OnCancel(self, event):
        """Cancel dialog"""
        self.EndModal(wx.ID_CANCEL)

    def Validate(self):
        """Dialog validation"""
        self._name = self.m_textCtrl6.GetValue().strip()
        if len(self._name) == 0:
            wx.MessageBox("Member name must not be empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^([A-Za-z_])[0-9A-Za-z_]*", self._name) is None:
            wx.MessageBox("Member name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        iscope = self.m_radioBox1.GetSelection()
        if iscope == 0:
            self._scope = 'class'
        else:
            self._scope = 'instance'
        self._value = self.m_textCtrl8.GetValue().strip()
        if len(self._value) == 0:
            self._value = "None"
        self._note = self.m_richText1.GetValue()
        return True

    def get_kwargs(self):
        """return arguments for object instance"""
        return  {'parent': self._container, 'name': self._name,
            'scope': self._scope, 'value': self._value,
            'note': self._note}

    def SetAttributes(self, member):
        """Setup attributes for editing already method"""
        self.SetTitle("Edit member")
        self.m_textCtrl6.SetValue(member._name)
        if member._scope == 'class':
            self.m_radioBox1.SetSelection(0)
        else:
            self.m_radioBox1.SetSelection(1)
        self.m_textCtrl8.SetValue(member._value)
        self.m_richText1.SetValue(member.note)

    def CopyAttributes(self, member):
        """Setup attributes"""
        member._name = self._name
        member._scope = self._scope
        member._value = self._value
        member.note = self._note

    def OnOK(self, event):
        """ok event handler"""
        if self.Validate():
            self.EndModal(wx.ID_OK)



"""Subclass of NewDecorator, which is generated by wxFormBuilder."""

import wx
import wxx
from activity.models.ui import ui as ui
import app.resources as rc


# Implementing NewDecorator
class PyDecoratorDialog(ui.NewDecorator):
    """Create or edit a decorator
    with this dialog"""
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """init dialog"""
        super(PyDecoratorDialog, self).__init__(parent)
        self._container = container
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("decorator"))
        self.SetIcon(icon)

    def CopyAttributes(self, decorator):
        """Get the atributes"""
        decorator._name = self._name
        decorator._call = self._call
        decorator.note = self._note

    def SetAttributes(self, decorator):
        """Set the attributes"""
        if type(decorator) is tuple:
            self._name = decorator[0]
            self._call = decorator[1]
            self._note = wx.EmptyString
        else:
            self._name = decorator._name
            self._call = decorator._call
            self._note = decorator.note
        self.m_textCtrl79.SetValue(self._name)
        if self._call:
            self.m_textCtrl80.SetValue(self._call)
        else:
            self.m_textCtrl80.SetValue(wx.EmptyString)
        #self.m_richText1.SetValue(decorator.note)
        self.SetTitle("Edit decorator")

    def Validate(self):
        """Validate dialog"""
        self._name = self.m_textCtrl79.GetValue().strip()
        if len(self._name) == 0:
            wx.MessageBox("decorator name missing", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._call = self.m_textCtrl80.GetValue().strip()
        self._note = wx.EmptyString
        return True

    def get_kwargs(self):
        """return arguments for object instance"""
        return  {'parent': self._container, 'name': self._name,
            'call': self._call, 'note': self._note}

    # Handlers for NewDecorator events.
    def OnCancel(self, event):
        """Handle cancel"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        "Handle Ok "
        if self.Validate():
            self.EndModal(wx.ID_OK)



"""Subclass of IsClassMethods, which is generated by wxFormBuilder."""

import wxx
from activity.models.ui import ui as ui
from app.utils import cached_type
import model


# Implementing IsClassMethods
class IsClassMethods(ui.IsClassMethods):
    """
    This dialog allows to add/remove is_class methods.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Dialog initialization"""
        super(IsClassMethods, self).__init__(parent)
        # container es la clase base
        self.container = container
        # create a map of feasible casts
        self._classes = []
        for k in container._deriv:
            self.visit(k)
        # get current methods
        self._is_class_methods = container(model.cc.IsClassMethod)
        # create map from names to implementations
        self._is_class_method_names = dict([(x._name, x) for x in self._is_class_methods])
        # create map from feasible is_class to current impl
        self._map = {}
        for k in self._classes:
            name = k.scoped
            name = "is_" + name.replace('::', '_')
            if name in self._is_class_method_names:
                self._map[name] = (k, self._is_class_method_names[name])
            else:
                self._map[name] = (k, None)
        # do a label insertion remembering state
        pos = 0
        for k in self._map:
            v = self._map[k]
            self.m_checkList2.Insert(k, pos, v)
            if v[1]:
                self.m_checkList2.Check(pos)
            pos = pos + 1

    def visit(self, k):
        """Add inheritance branch"""
        for l in k._deriv:
            self.visit(l)
        self._classes.append(k)

    def get_kwargs(self):
        """Returns kwargs dictionary suitable for objects creation"""
        kwargs_list = []
        tbool = cached_type(self.container.project, 'bool')
        for item in range(0, self.m_checkList2.GetCount()):
            v = self.m_checkList2.GetClientData(item)
            c = self.m_checkList2.IsChecked(item)
            if  (c and v[1]) or (not c and not v[1]):
                continue
            if c:
                kwargs = {}
                derivative = v[0]
                kwargs['parent'] = self.container
                kwargs['name'] = 'is_' + derivative.scoped.replace('::', '_')
                kwargs['type'] = model.cc.typeinst(
                        type=tbool, const=True)
                kwargs['constmethod'] = True
                kwargs['note'] = 'This method checks if the instance is specialized as {}'.format(derivative.GetFullLabel())
                kwargs['declare'] = True
                kwargs['implement'] = True
                kwargs['content'] = '\treturn ( dynamic_cast<const {}*>(this) != nullptr );'.format(name)
                kwargs_list.append(kwargs)
            else:
                v[1].Delete()
        return kwargs_list



"""Subclass of NewFile, which is generated by wxFormBuilder."""

import os
import wx
import wxx
import model
from activity.files.ui import ui as ui


# Implementing NewFile
class NewFile(ui.NewFile):
    """
    This dialog allows to create a new free file inside any project.
    A free file is a file that is not vincled with the project model
    an may be freely edited or modified.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """constructor"""
        self.container = container  # the receiver of the new element
        super(NewFile, self).__init__(parent)
        if type(container) is model.Project:
            self._path = self.container.dir
        else:
            self._path, f = os.path.split(self.container.abs_file)

    def Validate(self):
        """validate the dialog"""
        fname = self.m_file.GetValue().strip()
        if not fname:
            wx.MessageBox("File name must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._file = os.path.join(self._path, fname)
        #check about file existence
        if os.path.exists(self._file):
            wx.MessageBox("File already exists", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        return True

    def CopyAttributes(self, fobj):
        """Copy dialog selections to object"""
        fobj._file = self._file

    def SetAttributes(self, fobj):
        """Copy object attributes to dialog"""
        self._path, fname = os.path.split(fobj.abs_file)
        self.m_file.SetValue(fname)

    def get_kwargs(self):
        """Returns suitable arguments for creating object"""
        kwargs = {'parent': self.container, 'file': self._file}
        return kwargs

    def OnOK(self, event):
        """implement OnOK button"""
        if self.Validate():
            self.EndModal(wx.ID_OK)

    def OnCancel(self, event):
        """implement OnCancel button"""
        self.EndModal(wx.ID_CANCEL)





# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-

"""Git remote representation"""

import model
import app.resources as rc


class GitRemotes(model.TComponent):
    """Implements git remotes folder"""

    def __init__(self, **kwargs):
        """Initialization method"""
        self._lastHdrTime = None
        self._lastSrcTime = None
        if 'name' not in kwargs:
            kwargs['name'] = 'remotes'
        self._parent = kwargs['parent']
        super(GitRemotes, self).__init__(**kwargs)
        self.load_remotes()

    def load_remotes(self):
        """Update remotes"""
        kwargs = {'parent': self}
        for remote in self.repo._repo.remotes:
            kwargs['name'] = remote.name
            model.git.GitRemote(**kwargs)

    def update_status(self):
        """update the remote status"""
        old_remotes = dict([(x.name, x) for x in self(model.git.GitRemote)])
        new_remotes = dict([(x.name, x) for x in self.repo._repo.remotes])
        # remove missing remotes
        todel = [old_remotes[x] for x in old_remotes if x not in new_remotes]
        tonew = [new_remotes[x] for x in new_remotes if x not in old_remotes]
        for x in todel:
            x.Delete()
        kwargs = {'parent': self}
        for x in tonew:
            kwargs['name'] = x.name
            model.git.GitRemote(**kwargs)

    @property
    def repo(self):
        """get the local repo object"""
        return self.parent.repo

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {'data': self._data}
        kwargs.update(super(GitRemotes, self).get_kwargs())
        return kwargs

    @property
    def can_delete(self):
        """Check abot if class can be deleted"""
        return super(GitRemotes, self).can_delete

    def Delete(self):
        """Delete diagram objects"""
        super(GitRemotes, self).Delete()

    def RemoveRelations(self):
        """Utility for undo/redo"""
        super(GitRemotes, self).RemoveRelations()

    def RestoreRelations(self):
        """Utility for undo/redo"""
        super(GitRemotes, self).RestoreRelations()

    def SaveState(self):
        """Utility for saving state"""
        super(GitRemotes, self).SaveState()

    def OnUndoRedoRemoving(self, root=True):
        """Prepare object to delete"""
        super(GitRemotes, self).OnUndoRedoRemoving(root)

    def OnUndoRedoChanged(self):
        """Update from app"""
        #when the class is updated, ctor's and dtors, must be updated
        super(GitRemotes, self).OnUndoRedoChanged()

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(GitRemotes, self).OnUndoRedoAdd()

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('git_repo')

    @property
    def label(self):
        """Get tree label"""
        return '{self._name}'.format(self=self)






"""Subclass of SelectContexts, which is generated by wxFormBuilder."""

import wx
import wxx
from activity.models.ui import ui as ui


# Implementing SelectContexts
class SelectContexts(ui.SelectContexts):
    """
    This dialog allows to apply or remove
    contexts for an element.
    Applied contexts may insert prefix and
    sufix code both in element declaration
    an element implementation.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, obj):
        """Initialize dialog"""
        self.obj = obj
        self.contexts = obj.project._contexts
        if hasattr(obj, '_contexts'):
            self.applied = obj._contexts
        else:
            self.applied = []
        super(SelectContexts, self).__init__(parent)
        self.m_listCtrl5.InsertColumn(0, "context", format=wx.LIST_FORMAT_LEFT, width=-1)
        self.m_listCtrl4.InsertColumn(0, "context", format=wx.LIST_FORMAT_LEFT, width=-1)
        for c in self.contexts:
            if c not in self.applied:
                self.m_listCtrl5.Append((c._name,))
        for c in self.applied:
            self.m_listCtrl4.Append((c._name,))

    def CopyAttributes(self, obj):
        """Transfer context changes"""
        self.applied = []
        for i in range(0, self.m_listCtrl4.GetItemCount()):
            self.applied += [x for x in self.contexts if x._name == self.m_listCtrl4.GetItemText(i)]
        obj._contexts = self.applied

    def OnAddContext(self, event):
        """Handler for OnAddContext events."""
        i = self.m_listCtrl5.GetFirstSelected()
        if i == wx.NOT_FOUND:
            return
        s = self.m_listCtrl5.GetItemText(i)
        self.m_listCtrl5.DeleteItem(i)
        self.m_listCtrl4.Append((s,))
        pass

    def OnRemoveContext(self, event):
        """Handler for OnRemoveContext events."""
        i = self.m_listCtrl4.GetFirstSelected()
        while i != wx.NOT_FOUND:
            s = self.m_listCtrl4.GetItemText(i)
            self.m_listCtrl4.DeleteItem(i)
            self.m_listCtrl5.Append((s,))
            i = self.m_listCtrl4.GetFirstSelected()
        self.m_button31.Enable(False)
        self.m_button32.Enable(False)

    def OnSelectAssigned(self, event):
        """Handler for OnSelectAssigned events."""
        i = self.m_listCtrl4.GetFirstSelected()
        if i == wx.NOT_FOUND:
            self.m_button31.Enable(False)
            self.m_button32.Enable(False)
        else:
            if i + 1 < self.m_listCtrl4.GetItemCount():
                self.m_button32.Enable()
            else:
                self.m_button32.Enable(False)
            if i > 0:
                self.m_button31.Enable()
            else:
                self.m_button31.Enable(False)

    def OnMoveUp(self, event):
        """Move context up"""
        i = self.m_listCtrl4.GetFirstSelected()
        s = self.m_listCtrl4.GetItemText(i)
        self.m_listCtrl4.DeleteItem(i)
        self.m_listCtrl4.InsertStringItem(i - 1, s)

    def OnMoveDown(self, event):
        """Move context down"""
        i = self.m_listCtrl4.GetFirstSelected()
        s = self.m_listCtrl4.GetItemText(i)
        self.m_listCtrl4.DeleteItem(i)
        self.m_listCtrl4.InsertStringItem(i + 1, s)





"""Subclass of BuildBinaries, which is generated by wxFormBuilder."""

import app
import ctx


# Implementing BuildBinaries
class BuildBinaries(app.BuildBinaries):
    """Pane for binaries"""
    def __init__(self, parent):
        """Initialization"""
        super(BuildBinaries, self).__init__(parent)
        #read configuration
        self._config = ctx.theContext.config





# -*- coding: utf-8 -*-

import model.decorator as ctx
import app.resources as rc
import tran
import model


class MemberData(model.Member):
    """Implements member data"""
    context_container = True

    # visual methods
    @tran.TransactionalMethod('move member {}')
    def drop(self, to):
        """Drops datamember inside project or another folder """
        target = to.inner_member_container
        if not target or self.inner_class != target.inner_class or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        "Initialization"
        self._scope = kwargs.get('scope', "class")
        self._value = kwargs.get('value', "")
        super(MemberData, self).__init__(**kwargs)
        k = self.inner_module or self.inner_package
        if k:
            k.ExportPythonCodeFiles()


    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['scope'] = self._scope
        kwargs['value'] = self._value
        kwargs.update(super(MemberData, self).get_kwargs())
        return kwargs

    @ctx.ContextDeclaration()
    def WriteDeclaration(self, pf):
        """Write the member declaration"""
        pass

    @ctx.ContextImplementation()
    def WriteCode(self, pf):
        """Write the member implementation"""
        pass

    def GetInitializer(self):
        """Return the initializer sequence"""
        if len(self._value) > 0:
            return self._value
        return '{self._name}=None'.format(self=self)

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("py_member")

    def Delete(self):
        """Remove object"""
        k = self.inner_module or self.inner_package
        super(MemberData, self).Delete()
        if k:
            k.ExportPythonCodeFiles()

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(MemberData, self).OnUndoRedoAdd()

    def OnUndoRedoChanged(self):
        """Make changes in the model as result of change"""
        super(MemberData, self).OnUndoRedoChanged()

    def ExportPythonCode(self, wf):
        """Write code"""
        if len(self._value):
            wf.writeln('{self._name} = {self._value}'.format(self=self))
        else:
            wf.writeln('{self._name} = None'.format(self=self))



"""Subclass of NewArgument, which is generated by wxFormBuilder."""

import copy
import re
import wx
import wx.lib.newevent
import wxx
from activity.models.ui import ui as ui
import app.resources as rc
import model


# Implementing NewArgument
class ArgumentDialog(ui.NewArgument):
    """
    This dialog allows to specify name, type and attributes for an method
    argument. For the argument type, you can choose between known types
    or, if applicable, template type arguments determined by context.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, method):
        """Initialization. The parent argument represents the mainWindow """
        super(ArgumentDialog, self).__init__(parent)
        self._container = method
        self._types = dict([(x._name, x) for x in method.types])
        self._typei = model.cc.typeinst(type=self._types["int"])
        self._origin = None
        self._autoname = ''  # proposed name
        # add types but not on-the-fly template type
        self.m_type.AppendItems([x for x in self._types.keys() if x != '@'])

        #get nested template types (now easy)
        self._template_types = method.template_types
        if len(self._template_types) > 0:
            self.m_type.AppendItems(self._template_types)
        self.choiceStr = ""
        self.m_type.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("member"))
        self.SetIcon(icon)

    def AutoName(self):
        """Suggest the argument name, based on type"""
        iSel = self.m_type.GetCurrentSelection()
        if iSel == wx.NOT_FOUND:
            return
        s = self.m_name.GetValue()
        if self._autoname != s and s:
            return
        kwargs = {
            'const': (self.m_const.IsChecked() and 'c') or '',
            'reference': (self.m_reference.IsChecked() and 'r') or '',
            'ptr': (self.m_ptr.IsChecked() and 'p') or '',
            'pptr': (self.m_pptr.IsChecked() and 'p') or '',
            'constptr': (self.m_constptr.IsChecked() and 'c') or '',
            'array': (self.m_array.IsChecked() and 'a') or '',
            'typename': self.m_type.GetString(iSel),
        }
        #volatile = (self.m_volatile.IsChecked() and 'v') or ''
        self._autoname = '{const}{reference}{ptr}{pptr}{constptr}{array}{typename}'.format(
            **kwargs)
        self.m_name.SetValue(self._autoname)

    def OnTogglePtr(self, event):
        """Suggest argument name, if untouched"""
        if self.m_ptr.IsChecked():
            self.m_constptr.Enable(True)
            self.m_pptr.Enable(True)
        else:
            self.m_constptr.SetValue(False)
            self.m_pptr.SetValue(False)
            self.m_constptr.Enable(False)
            self.m_pptr.Enable(False)
        self.AutoName()

    def OnToggleConst(self, event):
        """Suggest argument name, if untouched"""
        self.AutoName()

    def OnToggleConstPtr(self, event):
        """Suggest argument name, if untouched"""
        self.AutoName()

    def OnTogglePointerPointer(self, event):
        """Suggest argument name, if untouched"""
        self.AutoName()

    def OnToggleArray(self, event):
        "toggle array event"
        if  self.m_array.IsChecked():
            self.m_textCtrl7.Show(True)
            self.m_textCtrl7.Enable(True)
        else:
            self.m_textCtrl7.Show(False)
            self.m_textCtrl7.Enable(False)
        self.AutoName()

    def OnToggleReference(self, event):
        """Suggest argument name, if untouched"""
        self.AutoName()

    def OnTypeChanged(self, event):
        """This event happens when the return type is changed. The main goal
        of this callback is handling template types for argument specification"""
        iSel = self.m_type.GetCurrentSelection()
        _type = self._types.get(self.m_type.GetString(iSel), None)
        template_args = False
        if _type is not None:
            if _type._template is not None:
                template_args = True
        if template_args is True:
            self.m_staticText67.Enable(True)
            self.m_template_args.Enable(True)
            self.m_staticText68.Enable(True)
        else:
            self.m_staticText67.Enable(False)
            self.m_template_args.Enable(False)
            self.m_staticText68.Enable(False)
            self.m_template_args.SetValue('')
        event.Skip()

    def CopyAttributes(self, argument):
        """Get the atributes"""
        argument._name = self._name
        argument._typei = copy.copy(self._typei)
        argument._default = self._default
        argument.note = self._note

    def SetAttributes(self, argument):
        """Set the attributes"""
        self._origin = argument._name
        self.m_name.SetValue(argument._name)
        ti = argument._typei
        iSel = self.m_type.FindString(ti.type_name)
        self.m_type.SetSelection(iSel)
        self.m_textCtrl8.SetValue(argument._default)
        self.m_const.SetValue(ti._const)
        self.m_ptr.SetValue(ti._ptr)
        self.m_reference.SetValue(ti._ref)
        self.m_pptr.SetValue(ti._ptr_to_ptr)
        self.m_constptr.SetValue(ti._const_ptr)
        self.m_array.SetValue(ti._array)
        if ti._array is True:
            self.m_textCtrl7.Show(True)
            self.m_textCtrl7.Enable(True)
            self.m_textCtrl7.SetValue(str(ti._array_size))
        else:
            self.m_textCtrl7.SetValue('0')
        if ti._type_args is not None:
            self.m_staticText67.Enable(True)
            self.m_template_args.Enable(True)
            self.m_staticText68.Enable(True)
            self.m_template_args.SetValue(ti._type_args)
        if self.m_ptr.IsChecked():
            self.m_constptr.Enable(True)
            self.m_pptr.Enable(True)
        self.m_richText1.SetValue(argument.note)
        self.SetTitle("Edit argument")

    def Validate(self):
        """Validation"""
        self._name = self.m_name.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Argument name must not be empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._name) is None:
            wx.MessageBox("Argument name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if  self._origin != self._name:
            if self._container.ExistArgumentNamed(self._name):
                wx.MessageBox("Argument name already exists", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        iSel = self.m_type.GetCurrentSelection()
        if iSel == wx.NOT_FOUND:
            wx.MessageBox("Invalid type", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        typename = self.m_type.GetString(iSel)
        if self.m_array.IsChecked():
            asize = int(self.m_textCtrl7.GetValue())
        else:
            asize = 0
        if typename in self._template_types:
            self._typei = model.cc.typeinst(
                type=self._types['@'],
                type_alias=typename,
                const=self.m_const.IsChecked(),
                ptr=self.m_ptr.IsChecked(),
                ref=self.m_reference.IsChecked(),
                ptrptr=self.m_pptr.IsChecked(),
                constptr=self.m_constptr.IsChecked(),
                array=self.m_array.IsChecked(),
                arraysize=asize)
        else:
            _type = self._types[typename]
            if _type._template is not None:
                #we construct type instance with explicit arguments
                type_args = self.m_template_args.GetValue()
                self._typei = model.cc.typeinst(
                    type=_type,
                    type_args=type_args,
                    const=self.m_const.IsChecked(),
                    ptr=self.m_ptr.IsChecked(),
                    ref=self.m_reference.IsChecked(),
                    ptrptr=self.m_pptr.IsChecked(),
                    constptr=self.m_constptr.IsChecked(),
                    array=self.m_array.IsChecked(),
                    arraysize=asize
                )
            else:
                self._typei = model.cc.typeinst(
                    type=self._types[typename],
                    const=self.m_const.IsChecked(),
                    ptr=self.m_ptr.IsChecked(),
                    ref=self.m_reference.IsChecked(),
                    ptrptr=self.m_pptr.IsChecked(),
                    constptr=self.m_constptr.IsChecked(),
                    array=self.m_array.IsChecked(),
                    arraysize=asize
            )
        self._default = self.m_textCtrl8.GetValue()
        self._note = self.m_richText1.GetValue()
        return True

    def get_kwargs(self):
        """returns arguments suitable for creating new object"""
        return {'parent': self._container, 'name': self._name,
            'type': self._typei, 'default': self._default,
            'note': self._note}

    # Handlers for NewArgument events.
    def OnKeyDown(self, event):
        """Listbox selection"""
        keycode = event.GetKeyCode()
        if keycode == wx.WXK_UP or keycode == wx.WXK_NUMPAD_UP:
            print("up\n")
            i = self.m_type.GetSelection()
            if i is not wx.NOT_FOUND and i > 0:
                self.m_type.SetSelection(i - 1)
        elif keycode == wx.WXK_DOWN or keycode == wx.WXK_NUMPAD_DOWN:
            print("down\n")
            i = self.m_type.GetSelection() + 1
            if i > wx.NOT_FOUND and i < len(self._types):
                self.m_type.SetSelection(i)
        elif keycode < 256:
            keychar = chr(keycode)
            if keychar.isalnum() or keycode is wx.WXK_SPACE:
                self.choiceStr += keychar.lower()
                for t in self._types:
                    tl = t.lower()
                    if tl.find(self.choiceStr) == 0:
                        #print "found str "+t+"from "+self.choiceStr+"\n"
                        sel = self.m_type.FindString(t)
                        if sel is not wx.NOT_FOUND:
                            self.m_type.SetSelection(sel)
                            if keycode is not wx.WXK_SPACE:
                                event.Skip()
                            return
            self.choiceStr = ""
        event.Skip()

    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """ok event handler"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





"""Subclass of OpenGitRepo, which is generated by wxFormBuilder."""


import wx
import wxx
from activity.git.ui import ui as ui
import app.resources as rc


# Implementing OpenGitRepo
class OpenGitRepoDialog(ui.OpenGitRepo):
    """
    This dialogs allows to select an existing
    repository and integrate it into the workspace

    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialize dialog"""
        super(OpenGitRepoDialog, self).__init__(parent)
        self.container = container
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("git"))
        self.SetIcon(icon)

    def get_kwargs(self):
        """Returns kwargs dictionary suitable for object creation"""
        kwargs = {}
        kwargs['parent'] = self.container
        kwargs['uri'] = self._path
        kwargs['local'] = True
        return kwargs

    def Validate(self):
        """Validate the dialog"""
        self._path = self.m_dirPicker2.GetPath().strip()
        if len(self._path) == 0:
            wx.MessageBox("Git directory must be specified", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        #really check git repo
        try:
            import git
            repo = git.Repo(self._path)
            if repo.bare:
                wx.MessageBox("The specified dir is a bare Git repository\nOnly working repositories are allowed.", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        except git.exc.InvalidGitRepositoryError:
            wx.MessageBox("The specified dir is not a Git repository", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        except git.exc.NoSuchPathError:
            wx.MessageBox("The specified path is invalid", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        return True

    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """Handle OnOk"""
        if self.Validate():
            self.EndModal(wx.ID_OK)



"""Subclass of Navigator, which is generated by wxFormBuilder."""

import wx
from activity.models.ui import ui as ui
import app.resources as rc
from model import cc


# Implementing Navigator
class NavigatorDialog(ui.Navigator):
    """
    This dialog shows insertable elements
    """
    _escKeyId = wx.Window.NewControlId()

    def __init__(self, parent, root, pos):
        """Initialice the navigator dialog"""
        super(NavigatorDialog, self).__init__(parent)
        self.imglist = rc.GetBitmapImageList()
        self.m_tree.SetImageList(self.imglist)
        # The root element is a class :
        # iterate and add elements
        treeid = self.m_tree.AddRoot(wx.EmptyString)
        self.m_tree.SetPyData(treeid, wx.TreeItemData(root))
        self.FillTree(treeid, root)
        first, cookie = self.m_tree.GetFirstChild(treeid)
        if first.IsOk():
            self.m_tree.SelectItem(first)
        self._set_accelerators()
        self.Move(pos)
        self.Show(True)
        self._str = wx.EmptyString

    def FillTree(self, treeid, obj, prefix=wx.EmptyString):
        """Fill the tree"""
        if obj is None:
            return
        if type(obj) is cc.Class:
            f = lambda x: x.inner_class == obj
            for member in obj(cc.MemberData, filter=f, cut=True):
                treeitemid = self.m_tree.AppendItem(treeid, '{prefix}{name}'.format(
                    prefix=prefix, name=member.prefixed_name), image=member.bitmap_index)
                if type(member.type_instance.base_type) is cc.Class:
                    # Add has child property
                    self.m_tree.SetItemHasChildren(treeitemid, True)
                self.m_tree.SetPyData(treeitemid, wx.TreeItemData(member))
            for function in obj(cc.MemberMethod, filter=f, cut=True):
                treeitemid = self.m_tree.AppendItem(treeid, '{prefix}{call}'.format(
                    prefix=prefix, call=function.call), image=function.bitmap_index)
                if type(function.type_instance.base_type) is cc.Class:
                    # Add has child property
                    self.m_tree.SetItemHasChildren(treeitemid, True)
                self.m_tree.SetPyData(treeitemid, wx.TreeItemData(function))

    def OnSelectElement(self, event):
        """Handle to ensure scroll"""
        self.m_tree.ScrollTo(event.GetItem())

    def OnExpandItem(self, event):
        """Handle the expansion of the tree and inserting elemens"""
        treeitem = event.GetItem()
        if not treeitem.IsOk():
            return
        obj = self.m_tree.GetPyData(treeitem).GetData()
        if obj is None:
            return
        if self.m_tree.GetChildrenCount(treeitem, False) > 0:
            return
        tti = obj.type_instance
        if type(tti.base_type) is not cc.Class:
            return
        if tti.is_ptr:
            prefix = '->'
        else:
            prefix = '.'
        self.FillTree(treeitem, tti.base_type, prefix)

    def _set_accelerators(self):
        """Set the accelerator table"""
        aTable = wx.AcceleratorTable([
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_ESCAPE, NavigatorDialog._escKeyId)
        ])
        self.SetAcceleratorTable(aTable)
        self.Bind(wx.EVT_MENU, self.OnCancel, id=NavigatorDialog._escKeyId)

    def OnLoseFocus(self, event):
        """Cancel search"""
        self.EndModal(wx.ID_CANCEL)

    def OnCancel(self, event):
        """Cancel search"""
        self.EndModal(wx.ID_CANCEL)

    def ChainedString(self, treeitem):
        if not treeitem.IsOk():
            return wx.EmptyString
        parentitem = self.m_tree.GetItemParent(treeitem)
        if not parentitem.IsOk():
            return wx.EmptyString
        text = self.m_tree.GetItemText(treeitem)
        return self.ChainedString(parentitem)+text

    @property
    def value(self):
        """return selected text"""
        return self._str

    def OnChoice(self, event):
        """Do final selection"""
        self._str = self.ChainedString(self.m_tree.GetSelection())
        self.EndModal(wx.ID_OK)



# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:08:46 2013

@author: mel
"""
from MemberMethod import MemberMethod
import tran


class IsClassMethod(MemberMethod):
    """Implements is_class method"""
    context_container = True
    argument_container = True

    # visual methods
    @tran.TransactionalMethod('move is_class method {}')
    def drop(self, to):
        """Drops datamember inside project or another folder """
        target = to.inner_member_container
        if not target or self.inner_class != target.inner_class or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization"""
        super(IsClassMethod, self).__init__(**kwargs)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs.update(super(IsClassMethod, self).get_kwargs())
        return kwargs

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(IsClassMethod, self).OnUndoRedoChanged()

    def OnUndoRedoRemoving(self, root=True):
        """Prepare for delete"""
        super(IsClassMethod, self).OnUndoRedoRemoving(root)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(IsClassMethod, self).OnUndoRedoAdd()



# -*- coding: utf-8 -*-

"""The mission of logger is create an action logger suitable for creating operations
logs that will be useful (for example, for documenting operations)
"""
import wx
import time
import os


LOG_START = False  # whether register or not
LOG_END = False  # whether register or not


class loggerEntry(object):
    """Represents a single logger entry"""
    def __init__(self, text, reference=None):
        """Initialize logger"""
        self._reference = reference
        self._text = text
        self._time = time.strftime('%d-%m-%Y %H:%M:%S')
        wx.LogMessage(text)
        super(loggerEntry, self).__init__()

    @property
    def reference(self):
        """Return the reference"""
        return self._reference

    @property
    def status(self):
        """Return the stack status"""
        import tran
        return tran.TransactionalStack.instance.TransactionStatus(self._reference)

    def __str__(self):
        '''Return an string representation of the entry'''
        return '[{self._time}] {self._text}'.format(self=self)


class logger(object):
    """Logger that holds the operations"""

    def __init__(self, **kwargs):
        """Initialize logger"""
        self._log = []
        self._backlog = []
        self._filter = kwargs.get('filter', None)  # the filter
        self._fname = None  # the file name
        self._queue = []  # next logger queue
        super(logger, self).__init__()

    def pushLogger(self, another):
        """Append another logger"""
        self._queue.append(another)

    def popLogger(self, another):
        """Remove logger from queue"""
        if another in self._queue:
            i = self._queue.index(another)
            del self._queue[i]

    def addEntry(self, text, reference=None):
        """Append new log entry"""
        self._backlog = []
        self._log.append(loggerEntry(text, reference))

    def __call__(self, command, transaction=None):
        """Interface for transaction handler"""
        if command == 'start':
            self._log = []
            if LOG_START:
                self.addEntry('====== STARTING LOG ======')
            else:
                self._backlog = []
            return
        if command == 'commit':
            if self._filter is None or self._filter(transaction):
                self.addEntry(transaction._name, hash(object))
                #propagate
                for log in self._queue:
                    log(command, transaction)
        elif command == 'undo':
            assert(len(self._log) > 0)
            entry = self._log.pop()
            self._backlog.append(entry)
            #propagate
            for log in self._queue:
                if not len(log._log):
                    continue
                if entry.reference() == log._log[-1].reference:
                    log('undo')
        elif command == 'redo':
            assert(len(self._backlog) > 0)
            entry = self._backlog.pop()
            self._log.append(entry)
            for log in self._queue:
                if not len(log._backlog):
                    continue
                if entry.reference() == log._backlog[-1].reference:
                    log('redo')
        elif command == 'end':
            self._log = []
            if LOG_END:
                self.addEntry('====== ENDING LOG ======')
            else:
                self._backlog = []

    def SetFile(self, fname):
        """Sets the log file"""
        self._fname = fname

    def Dump(self):
        """This command only dumps current logs (not backlogs)"""
        #propagate
        for log in self._queue:
            log.Dump()
        if not self._fname:
            return False
        try:
            fname = os.path.realpath(self._fname)
            f = open(fname, 'w+')
            for i in range(0, len(self._log)):
                f.write('{logline}\n'.format(logline=str(self._log[i])))
            f.close()
        except:
            return False





# -*- coding: utf-8 -*-
"""
Created on Fri July 3 20:50:32 2015

@author: mel
"""
import model
import model.decorator as ctx
import tran
from app import resources
from tran import TransactionStack


class Namespace(model.TComponent):
    """Implements class representation"""

    class_container = True
    folder_container = True
    diagram_container = True
    module_container = True
    namespace_container = True
    function_container = True
    variable_container = True

    #visual methods
    @tran.TransactionalMethod('move namespace {}')
    def drop(self, to):
        """drop this elemento to another place"""
        target = to.inner_namespace_container
        if not target or to.project != self.project:
            return False  # avoid move arguments between projects
        index = 0  # trick for insert as first child
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization method"""
        self._lastSrcTime = None
        self._lastHdrTime = None
        super(Namespace, self).__init__(**kwargs)
        self.update_container()
        k = self.outer_class or self.outer_module or self.project
        if k:
            k.ExportCppCodeFiles(force=True)

    def Delete(self):
        """Handle delete"""
        k = self.outer_class or self.outer_module or self.project
        super(Namespace, self).Delete()
        if k:
            k.ExportCppCodeFiles(force=True)


    def update_container(self):
        """Update the container info"""
        if self.inner_module:
            self.class_container = False
            self.module_container = False
            self.function_container = True
            self.variable_container = True
        else:
            self.class_container = True
            self.module_container = True
            self.function_container = False
            self.variable_container = False

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs.update(super(Namespace, self).get_kwargs())
        return kwargs

    def lower(self):
        """Criteria for sorting when generating code"""
        return self._inheritance_level

    @ctx.ContextDeclaration()
    def WriteDeclaration(self, pf):
        """Write the folder declaration"""
        #we will follow and respect the visual order
        self.WriteComment(pf)
        child_types = [model.Folder, model.cc.Namespace,
            model.cc.Class, model.cc.Enum, model.cc.MemberData,
            model.cc.Constructor, model.cc.MemberMethod,
            model.cc.Destructor, model.cc.Module,
            model.cc.Function, model.cc.Data]
        pf.openbrace('namespace {self._name}'.format(self=self))
        for type in child_types:
            if len(self[type]):
                for item in self[type]:
                    item.WriteDeclaration(pf)
        pf.closebrace(';')

    @property
    def can_delete(self):
        """Chack bout if this object may be deleted"""
        return super(Namespace, self).can_delete

    def RemoveRelations(self):
        """Utility for undo/redo"""
        super(Namespace, self).RemoveRelations()

    def RestoreRelations(self):
        """Utility for undo/redo"""
        super(Namespace, self).RestoreRelations()

    def OnUndoRedoRemoving(self, root=True):
        """Prepare object to delete"""
        super(Namespace, self).OnUndoRedoRemoving(root)

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(Namespace, self).OnUndoRedoChanged()
        if not TransactionStack.InUndoRedo():
            k = self.outer_class or self.outer_module
            if k:
                k.ExportCppCodeFiles(force=True)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(Namespace, self).OnUndoRedoAdd()

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return resources.GetBitmapIndex('namespace')

    @property
    def tree_label(self):
        """Get tree label"""
        return 'namespace {namespace._name}'.format(namespace=self)

    @property
    def nested_classes(self):
        """Returns the list of nested classes (including self)"""
        if type(self.parent) in [model.Folder, model.cc.Class, model.Namespace]:
            return self.parent.nested_classes
        return []

    @property
    def scoped(self):
        """Get the scope"""
        return '{self.parent.scope}{self._name}'.format(self=self)

    @property
    def scope(self):
        """Get the scope"""
        return '{scoped}::'.format(scoped=self.scoped)



"""Subclass of ChooseFile, which is generated by wxFormBuilder."""


import os
import wx
import wxx
import Tool
import ast
import analytic.astd as astd


# Implementing ChooseFile
class ToolChooseFile(Tool.ChooseFile):
    """
    Use this dialog for finding the python file
    you want to explore in ast explorer.
    The ast explorer will show the python abstract
    syntax tree.
    Please pay attention to the fact that the ast
    structure may change, so you must take this
    in consideration, if you want to use ast for
    your own purposes.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent):
        """init dialog"""
        Tool.ChooseFile.__init__(self, parent)

    def Validate(self):
        """Test if the dialog is ok"""
        #get the selected file
        self._path = self.m_filePicker1.GetPath().strip()
        if len(self._path) == 0:
            wx.MessageBox("Please choose any file", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        #check for readability
        if not os.access(self._path, os.F_OK | os.R_OK):
            wx.MessageBox("The file is not readable", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        #check the file contents
        try:
            with open(self._path, "r") as python_file:
                content = python_file.read()
                self._tree = astd.parse(content)
        except:
            wx.MessageBox("The file contains errors", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        return True

    def OnOK(self, event):
        """Process Ok button"""
        if self.Validate():
            self.EndModal(wx.ID_OK)




import re
import wx
import wxx
from activity.models.ui import ui as ui
import app.resources as rc
import model.py


# Implementing NewMethod
class PyMemberMethodDialog(ui.NewPyMemberMethod):
    """
    This dialog allows to setup the attributes
    of a member function of python class.
    You can configure the method for not
    generating code at all.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialization"""
        super(PyMemberMethodDialog, self).__init__(parent)
        self.container = container
        self.m_comboBox5.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("py_method"))
        self.SetIcon(icon)

    def Validate(self):
        """Validation"""
        self._name = self.m_comboBox5.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Method name must not be empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._name) is None:
            wx.MessageBox("Method name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._classmethod = self.m_checkBox21.IsChecked()
        self._staticmethod = self.m_checkBox6.IsChecked()
        self._property = self.m_checkBox41.IsChecked()
        self._note = self.m_richText1.GetValue()
        self._implement = self.m_checkBox86.IsChecked()
        return True

    def get_kwargs(self):
        """Returns kwargs dictionary suitable for object creation"""
        kwargs = {}
        kwargs['parent'] = self.container
        kwargs['name'] = self._name
        kwargs['class_method'] = self._classmethod
        kwargs['static_method'] = self._staticmethod
        kwargs['property'] = self._property
        kwargs['note'] = self._note
        kwargs['implement'] = self._implement
        return kwargs

    def CopyAttributes(self, method):
        """Copy attributes to method"""
        method._name = self._name
        if self._classmethod:
            if not method._classmethod:
                model.py.Decorator(parent=method, name='classmethod')
        elif method._classmethod:
            method._classmethod.Delete()
        if self._staticmethod:
            if not method._staticmethod:
                model.py.Decorator(parent=method, name='staticmethod')
        elif method._staticmethod:
            method._staticmethod.Delete()
        if self._property:
            if not method._property:
                model.py.Decorator(parent=method, name='property')
        elif method._property:
            method._property.Delete()
        method._note = self._note
        method._implement = self._implement

    def SetAttributes(self, method):
        """Setup attributes for editing already method"""
        self.m_comboBox5.SetValue(method._name)
        self.m_checkBox6.SetValue(bool(method._staticmethod))
        self.m_checkBox21.SetValue(bool(method._classmethod))
        self.m_checkBox41.SetValue(bool(method._property))
        self.m_richText1.SetValue(method.note)
        import model.py
        if type(method) is model.py.InitMethod:
            self.SetTitle("Edit __init__ method")
            self.m_comboBox5.Enable(False)
            self.m_checkBox21.Enable(False)
            self.m_checkBox6.Enable(False)
            self.m_checkBox41.Enable(False)
            icon = wx.EmptyIcon()
            icon.CopyFromBitmap(rc.GetBitmap("py_init"))
            self.SetIcon(icon)
        else:
            self.SetTitle("Edit method")
            if len(method[model.py.Argument]) > 1 or method[model.py.ArgsArgument] or method[model.py.KwArgsArgument]:
                self.m_checkBox41.Enable(False)

    # Handlers for NewMethod events.
    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """ok event handler"""
        if self.Validate():
            self.EndModal(wx.ID_OK)


